#ifndef JRPLUGINRCW
#define JRPLUGINRCW

using namespace System;
using namespace System::Reflection;
using namespace System::IO;
using namespace System::Runtime::InteropServices;
using namespace System::Collections::Generic;
using namespace System::Xml::XPath;

#include "JrPluginFuncs.h"

namespace FARRPlugin
{

/// <summary>
/// The base class from which you inherit your C# plugins
/// </summary>
public ref class FARRPluginBase abstract
{
public:

#pragma region Enums


	/// <summary>
	/// Global search state.
	/// </summary>
	enum class ESearchState
	{
		Stopped = E_SearchState_Stopped,
		Searching = E_SearchState_Searching
	};

	/// <summary>
	/// The kind of items in result state.
	/// Notice that not all item types are currently implemented.
	/// </summary>
	enum class EResultsAvailableState
	{
		/// <summary>
		/// No results in state
		/// </summary>
		None = E_ResultAvailableState_None,

		/// <summary>
		/// Standard item results.
		/// </summary>
		ItemResults = E_ResultAvailableState_ItemResuls,

		/// <summary>
		/// Displays text instead of the items. For example see the FCalc plugin or the showfile alias.
		/// </summary>
		DisplayText = E_ResultAvailableState_DisplayText,
		DisplayHtml = E_ResultAvailableState_DisplayHtml, 
		Grid = E_ResultAvailableState_Grid, 
		WinTakeover = E_ResultAvailableState_WinTakeover
	};


	/// <summary>
	/// The type of the result item.
	/// </summary>
	enum class EEntryType
	{
		Unknown = E_EntryType_UNKNOWN,
		File = E_EntryType_FILE,
		Folder = E_EntryType_FOLDER,
		Alis = E_EntryType_ALIAS,
		Url = E_EntryType_URL,
		Plugin = E_EntryType_PLUGIN,
		Clip = E_EntryType_CLIP
	};

	/// <summary>
	/// How the result is handled
	/// </summary>
	enum class EResultPostProcessing
	{
		/// <summary>
		/// If we have done the filtering and the result should be displayed
		/// </summary>
		ImmediateDisplay = E_ResultPostProcessing_ImmediateDisplay, 
		
		/// <summary>
		/// If the host should evaluate the result and add a score to it but still be forced to display it
		/// </summary>
		AddScore = E_ResultPostProcessing_AddScore, 

		/// <summary>
		/// If the host should evaluate the result and decide whether to show it based on match of search words
		/// </summary>
		MatchAgainstSearch = E_ResultPostProcessing_MatchAgainstSearch, 
		AddScore_wPats = E_ResultPostProcessing_AddScore_wPats , 
		MatchAgainstSearch_wPats = E_ResultPostProcessing_MatchAgainstSearch_wPats
	};

	/// <summary>
	/// The features this plugin is requesting from FARR
	/// </summary>
	enum class EWantFeatures
	{
		/// <summary>
		/// Do we want to search on regular expression matches
		/// </summary>
		SearchInputRegex = E_WantFeatures_searchinput_regex,
		/// <summary>
		/// Do we want to search on alias match
		/// </summary>
		SearchInputExplicit = E_SupportFeatures_searchinput_explicit,
		/// <summary>
		/// Do we want to search on all search expressions
		/// </summary>
		SearchInputAll =  E_SupportFeatures_searchinput_all,
		/// <summary>
		/// Do we want to try to handle triggers when a result is launched
		/// </summary>
		TryHandleTrigger = E_SupportFeatures_tryhandle_trigger,
		/// <summary>
		/// Do we want to try to handle triggers when a result is launched
		/// </summary>
		AddInfoFiles = E_SupportFeatures_addinfo_files, 
		/// <summary>
		/// Do we want to try to score EVERY file considered (SLOWS DOWN SEARCH!)
		/// </summary>
		ScoreAllFiles = E_SupportFeatures_scoreall_files, 
		/// <summary>
		/// Do we want to try to score files after other scoring has determined they are a viable result
		/// </summary>
		ScoreGoodFiles = E_SupportFeatures_scoregood_files, 
		/// <summary>
		/// Do we want our results to dominate and hide any others
		/// </summary>
		DominateResults = E_SupportFeatures_dominate_results

	};

#pragma endregion

	/// <summary>
	/// Initializing. You can initialize your plugin here.
	/// </summary>
	FARRPluginBase()
	{
		System::AppDomain::CurrentDomain->UnhandledException += gcnew UnhandledExceptionEventHandler(this, &FARRPluginBase::CurrentDomain_UnhandledException);

		// Initialize string data
		m_DefaultRegex = "";
		m_DefaultRegexFilter = "";
		m_DefaultKeyword = "";
		m_DefaultScore = "";
		m_DefaultAlias = "";

		// The general info parameters
		m_DisplayName = "";
		m_VersionString = "";
		m_ReleaseDateString = "";
		m_Author = "";
		m_HomepageURL = "";
		m_UpdateURL = "";
		m_ShortDescription = "";
		m_LongDescription = "";
		m_AdvConfigString = "";
		m_ReadMeString = "";
		m_IconFilename = "";

		m_DCUpdateFile = "";

		m_SearchState = ESearchState::Stopped;
		m_ResultsAvailableState = EResultsAvailableState::None;
		m_ResultsAmount = 0;

		m_WantFeautresDictionary = gcnew Dictionary<EWantFeatures, bool>();

		Array^ FeaturesValues = Enum::GetValues(EWantFeatures::typeid);
		for(int i = 0; i < FeaturesValues->Length; i++)
		{
			m_WantFeautresDictionary->Add((EWantFeatures)i, false);
		}


		SetSearchParameters(m_DefaultAlias, m_DefaultRegex, m_DefaultRegexFilter, m_DefaultKeyword, m_DefaultScore);
		SetFeatureParamaters(m_WantFeautresDictionary);
		SetGeneralInfoParameters(m_DisplayName, m_VersionString, m_ReleaseDateString, m_Author, 
			m_HomepageURL, m_UpdateURL, m_ShortDescription, m_LongDescription, 
			m_AdvConfigString, m_ReadMeString, m_IconFilename); 
	}

	
	virtual Boolean Init()
	{
		return true;
	}

	/// <summary>
	/// <para>This function is called by the host when the Plugin DLL is about to be unloaded.
	///  You cannot refuse it.</para>
	
	/// <para><b>Note:</b> You might release any resources you allocated previously here, or
	///  if you use visual forms, now would be the place to delete them.</para>
	/// </summary>
	/// <returns>Return false on error, true on success</returns>
	virtual Boolean Shutdown()
	{
		return true;
	}

	/// <summary>
	/// This function is called by the host when user pushes button to
	///  access ADVANCED options of the plugin.
	/// So you might here launch a windows dialog with advanced options, etc.
	/// <para><b>Note:</b> If your plugin supports advanced options, you have to return
	///  a non "" value for ThisPlugin_AdvConfigString</para>
	/// <para><b>Note:</b> It is your responsibility to save and load your advanced options
	///  though i expect to add a few callback functions for saving simple
	///  string and int values for you.</para>
	/// </summary>
	/// <returns>Return false on error, true on success</returns>
	virtual Boolean AdvConfig()
	{
		return true;
	}

	/// <summary>
	/// This function is called by the host when user pushes button to
	///  access the readme or help file options of the plugin.
	/// So you might here launch a text file or help file and let shell display
	///
	/// <para><b>Note:</b> If your plugin supports advanced options, you have to return
	///  a non "" value for ThisPlugin_ReadMeString</para>
	/// </summary>
	/// <returns>Return false on error, true on success</returns>
	virtual Boolean ShowReadMe()
	{
		return true;
	}

	/// <summary>
	/// Host calls this before iterating through the result requests
	/// In this way the dll should refrain from altering the results until retrieval is done
	/// One way to do this for asynchronous searching is to MOVE results to a return store
	/// on TRUE lock, and then free on FALSE.
	/// </summary>
	/// <returns>Return false on error, true on success</returns>
	virtual Boolean LockResults() { return true; }

	/// <summary>
	/// Host calls this after iterating through the result requests
	/// <para><b>Note:</b> When this function is called it is usually after FARR finished retrieving all your results and thus you will probably want to clear your results with <see cref="ClearResults"/></para>
	/// </summary>
	/// <returns>Return false on error, true on success</returns>	
	virtual Boolean UnlockResults() abstract;
	
	/// <summary>
	/// Host informs us that our window is about to appear
	/// </summary>
	/// <param name="hWnd">The window handle</param>
	virtual void WindowIsHiding(IntPtr hWnd)
	{
	}

	/// <summary>
	/// Host informs us that our window is about to disappear
	/// </summary>
	/// <param name="hWnd">The window handle</param>
	virtual void WindowIsUnHiding(IntPtr hWnd)
	{
	}


	/// <summary>
	/// Host informs us that a search is begining and what the searchstring is
	/// The Plugin DLL could do all "searching" here (if there is indeed any to do)
	/// And compute all results within this function, OR simply begin a threaded asynchronous search now.
	///
	/// <para>Normally the string you care about is searchstring_lc_nokeywords, thats the search string with all special +modifier keywords removed, and
	///  the alias keyword for this plugin removed, so all thats left is the string that should effect the results of the search.</para>
	///
	/// <para><b>NOTE</b>: if asynchronous searching is to be done, make sure to only set current_searchstate=E_SearchState_Stopped when done!</para>
	/// </summary>
	/// <param name="SearchStringRaw">The whole search string</param>
	/// <param name="SearchStringKeyKeywords">Just the keywords</param>
	/// <param name="ExplicitInvocation">True if the exact plugin alias was entered</param>
	/// <returns>returns TRUE only if the plugin decides now that no more searching should be done by any other plugin or builtin</returns>
	virtual Boolean SearchBegins(String^ SearchStringRaw, String^ SearchStringKeyKeywords, Boolean ExplicitInvocation) abstract;


	/// <summary>
	/// Host informs us that regular expression match **HAS** occured, and to begin doing that we need to do on such a match
	/// The Plugin DLL could do all "searching" here (if there is indeed any to do)
	/// And compute all results within this function, OR simply begin a threaded asynchronous search now.
	///
	/// <param>To access the group capture strings, use RegexGroups[1] to match the first group, etc.</param>
	/// <param>NOTE: if asynchronous searching is to be done, make sure to only set current_searchstate=E_SearchState_Stopped when done!</param>
	///
	/// </summary>
	/// <param name="SearchStringRaw">The whole search string</param>
	/// <param name="SearchStringKeyKeywords">Just the keywords</param>
	/// <param name="RegexGroupsCount">How many groups were matched in RegEx</param>
	/// <param name="RegexGroups">The matching groups array</param>
	/// <returns>returns TRUE only if the plugin decides now that no more searching should be done by any other plugin or builtin</returns>
	virtual Boolean RegexSearchMatch(String^ SearchStringRaw, String^ SearchStringKeyKeywords, int RegexGroupsCount, array<String^>^ RegexGroups)
	{
		return false;
	}


	/// <summary>
	/// Host informs us that a search has completed or been interrupted, and we should stop any processing we might be doing assynchronously
	/// You you will probably want to clear your results with <see cref="ClearResults"/>.
	/// NOTE: make sure to set current_searchstate=E_SearchState_Stopped;
	/// </summary>
	virtual void SearchEnds() abstract;

	/// <summary>
	/// Host is requesting one of our results.
	/// </summary>
	/// <param name="ResultIndex">The index of the item FARR requests</param>
	/// <param name="Path">The path of the item. You can pass any data but notice that it will appear in FARR status bar when the item is selected</param>
	/// <param name="Caption">The caption that will appear on the result item. Should be user friendly.</param>
	/// <param name="Group"></param>
	/// <param name="IconFile">A path to an ico file or an exe with an icon</param>
	/// <param name="ResultPostProcessing"><see cref="EResultPostProcessing"/></param>
	/// <param name="ScoreChange">By how much to adjust the score, can be positive or negative. You usually won't need to do it so you may not assign a value at all</param>
	/// <param name="EntryType">The type of your item, see <see cref="EEntryType"/></param>
	virtual void GetItemResultByIndex(int ResultIndex, String^% Path, String^% Caption, String^% Group, String^% IconFile, 
		EResultPostProcessing% ResultPostProcessing, int% ScoreChange, EEntryType% EntryType) abstract;

	/// <summary>
	/// Host is asking us if we want to modify the score of an item
	/// Original score in in *score, we can now modify it if we want
	/// </summary>
	/// <param name="Path">The path of the item</param>
	/// <param name="Score">By how much to adjust the score, can be positive or negative. You usually won't need to do it so you may not assign a value at all</param>
	/// <returns>Return TRUE if we modified it, or FALSE if not</returns>
	virtual Boolean AdjustResultScore(String^ Path, int% Score)
	{
		return false;
	}


	/// <summary>
	/// Host is calling us with a result item about to be triggered
	/// We can choose here to takeover the launching of it, OR simply do something and let normal launching continue
	/// <para>NOTE: It is adviced to check that the PluginId equals to ThisPluginId, otherwise we might be executing items that we never created</para>
	/// </summary>
	/// <param name="Path">Path of activated item</param>
	/// <param name="Caption">Caption of activated item</param>
	/// <param name="Group"></param>
	/// <param name="PluginId">The plugin id of activated item</param>
	/// <param name="ThisPlugId">The plugin of this plugin</param>
	/// <param name="Score">The score that FARR assigned to this item</param>
	/// <param name="EntryType">The type of activated item, see <see cref="EEntryType"/></param>
	/// <param name="CloseAfterOperation"></param>
	/// <returns>TRUE if activated the item, FALSE otherwise</returns>
	virtual Boolean ProcessTrigger(String^ Path, String^ Caption, String^ Group, 
		int PluginId, int ThisPlugId, int Score, 
		EEntryType EntryType, [Out] Boolean% CloseAfterOperation) abstract;

	// Configuring
	/// <summary>The alias that will trigger this plugin</summary>
	property String^ DefaultAlias { String^ get() { return m_DefaultAlias; } }

	/// <summary>Regular expresion that will trigger this plugin</summary>
	property String^ DefaultRegex { String^ get() { return m_DefaultRegex; } }
	property String^ DefaultRegexFilter { String^ get() { return m_DefaultRegexFilter; } }
	property String^ DefaultKeyword { String^ get() { return m_DefaultKeyword; } }
	/// <summary>Default score of this plugin items</summary>
	property String^ DefaultScore { String^ get() { return m_DefaultScore; } }
	// The general info parameters
	/// <summary>User friendly name of plugin</summary>
	property String^ DisplayName { String^ get() { return m_DisplayName; } }
	/// <summary>Version number</summary>
	property String^ VersionString { String^ get() { return m_VersionString; } }
	property String^ ReleaseDateString { String^ get() { return m_ReleaseDateString; } }
	property String^ Author { String^ get() { return m_Author; } }
	property String^ HomepageURL { String^ get() { return m_HomepageURL; } }
	/// <summary>Version number</summary>
	property String^ UpdateURL { String^ get() { return m_UpdateURL; } }
	property String^ ShortDescription { String^ get() { return m_ShortDescription; } }
	property String^ LongDescription { String^ get() { return m_LongDescription; } }
	property String^ AdvConfigString { String^ get() { return m_AdvConfigString; } }
	property String^ ReadMeString { String^ get() { return m_ReadMeString; } }
	/// <summary>Icon filename for plugin?  We pass icons by icon filename</summary>
	property String^ IconFilename { String^ get() { return m_IconFilename; } }


	/// <summary>
	/// Possible options:
	///
	///launch, LAUNCHFILENAME
	///statusbar, TEXTTOSETONSTATUSBAR
	///setsearch, TEXTTOSETINSEARCHEDIT
	///setsearchnogo, TEXTTOSETINSEARCHEDIT
	///stopsearch
	///window.hide
	///window.show
	///window.toggle
	///window.richeditmode, TEXTOPUTINRICHEDITWINDOW
	///window.richeditheight, HEIGHTOFWINDOW
	///window.richeditwidth, WIDTHOFWINDOW
	///setshowallmode
	///exit
	///reporterror, ERRORTEXTTOREPORTINPLUGINLOG
	///clipboard
	///PasteClipboardToLastActiveWindow,[TEXTTOSET] - paste the current clipboard text to the last active program (if value passed is not "" then clipboard will first be set to value.
	/// </summary>
	/// <param name="Command">The command</param>
	/// <param name="Argument">The pussed argument. Pass an emptry string if none is needed</param>
	void ExecuteFARRCommand(String^ Command, String^ Argument)
	{
		callbackfp_set_strval(hostptr,(char*)Marshal::StringToHGlobalAnsi(Command).ToPointer(), 
									  (char*)Marshal::StringToHGlobalAnsi(Argument).ToPointer());
	}

	void SetSearch(String^ Search)
	{
		ExecuteFARRCommand("setsearch", Search);
	}

protected:

	/// <summary>
	/// Notifies FARR about search state changes.
	/// <para>You should call this function whenever you change the <see cref="m_SearchState"/> or <see cref="m_ResultsAmount"/>.</para>
	/// </summary>
	void SearchStateChanged()
	{
		// tell the host that our state or resultcount has changed
		if (callbackfp_notifysearchstatechanged)
		{
			callbackfp_notifysearchstatechanged(hostptr,m_ResultsAmount,(E_SearchStateT)m_ResultsAvailableState);
		}
	}

	/// <summary>
	/// Sets plugin invocation parameters, see <see cref="FARRPluginBase"/> properties for reference.
	/// </summary>
	virtual void SetSearchParameters(String^% m_DefaultAlias, String^% DefaultRegex, String^% m_DefaultRegexFilter, 
		String^% m_DefaultKeyword, String^% m_DefaultScore) abstract;

	/// <summary>
	/// Sets plugin feature parameters, see <see cref="EWantFeatures"/> for reference.
	/// Set the items of the features you want to TRUE.
	/// </summary>
	virtual void SetFeatureParamaters(Dictionary<EWantFeatures, bool>^ FeatureDictionary) abstract;

	/// <summary>
	/// Sets plugin general info parameters, see <see cref="FARRPluginBase"/> properties for reference.
	/// </summary>
	virtual void SetGeneralInfoParameters(String^% DisplayName, String^% VersionString, String^% m_ReleaseDateString, String^% Author, 
		String^% HomepageURL, String^% UpdateURL, String^% m_ShortDescription, String^% LongDescription, 
		String^% AdvConfigString, String^% ReadMeString, String^% IconFilename) abstract; 


	/// <summary>
	/// The search state of the plugin. It would usually change on <see cref="SearchBegins"/> and <see cref="SearchEnds"/>.
	/// Whenever you update it you should usually also call <see cref="SearchStateChanged"/>.
	/// </summary>
	ESearchState m_SearchState;
	/// <summary>
	/// The state of the result items. It would usually change on <see cref="SearchBegins"/> and <see cref="SearchEnds"/>.
	/// </summary>
	EResultsAvailableState m_ResultsAvailableState;
	/// <summary>
	///  The amount of found items. It would usually change on <see cref="SearchBegins"/> and <see cref="SearchEnds"/>.
	/// Whenever you update it you should usually also call <see cref="SearchStateChanged"/>.
	/// </summary>
	int m_ResultsAmount;


	/// <summary>
	/// Helper enum that is used to get values from the DCUpdate file using the <see cref="GetValueFromDCUpdate"/> function.
	/// </summary>
	enum class EDCUpdateValues
	{
		IconFile,
		Version,
		WebPage,
		UpdateFile,
	};

	
	/// <summary>
	/// Returns the value from DCUpdate file
	/// If the DCUpdate file isn't found or the type isn't found, it throws an exception.
	/// </summary>
	/// <param name="Value">The type of the value to get</param>
	/// <returns>Returns the value</returns>
	String^ GetValueFromDCUpdate(EDCUpdateValues Value)
	{
		if(m_DCUpdateFile == "")
		{
			String^ AssemblyFolder = System::IO::Path::GetDirectoryName(System::Reflection::Assembly::GetExecutingAssembly()->Location) + "\\";
			array<String^>^ Files = System::IO::Directory::GetFiles(AssemblyFolder, "*.dcupdate");
			if (Files->Length == 1)
			{
				m_DCUpdateFile = Files[0];
			}
			else
			{
				throw gcnew Exception("Invalid amount of DCUPDATE files were found. Expected 1 and found: " + Files.Length.ToString());
			}
		}

		XPathDocument^ xdoc = gcnew XPathDocument(m_DCUpdateFile);
		XPathNavigator^ nav = xdoc->CreateNavigator();
		XPathNodeIterator^ nodeItor = nav->Select("Local/" + Value.ToString());
		if(nodeItor->Count == 0)
		{
			throw gcnew Exception("Specefied value ("+Value.ToString()+") is not found in your DCUpdate file. Redownloading this plugin might solve the problem: ");
		}

		nodeItor->MoveNext();
		return nodeItor->Current->Value->ToString();
	}


	/// <summary>
	/// <para>This helper function clears the number of the results <see cref="m_ResultsAmount"/> and sets <see cref="m_ResultsAvailableState"/> to none.</para>
	/// <para>It will be usually called after <see cref="SearchEnds"/> and <see cref="UnlockResults"/></para>
	/// </summary>
	void ClearResults()
	{
		m_ResultsAmount = 0;
		m_ResultsAvailableState = EResultsAvailableState::None;
	}

internal:

	// General lock request (lock or unlock)
	Boolean Request_LockResults(Boolean dolock) 
	{
		if(dolock)
		{
			return LockResults();
		}
		else
		{
			return UnlockResults();
		}
	}
	
	property ESearchState SearchState { ESearchState get() { return m_SearchState; } }
	property EResultsAvailableState ResultsAvailableState { EResultsAvailableState get() { return m_ResultsAvailableState; } }
	property int ResultsAmount { int get() { return m_ResultsAmount; } }







	/// <summary>
	/// Host is asking us if we want a certain feature or call
	/// </summary>
	/// <param name="Feature">Returns TRUE or FALSE depending on whether we want the feature. False is also returned if we fall through.</param>
	/// <returns></returns>
	virtual Boolean WantFeature(EWantFeatures Feature)
	{
		if(m_WantFeautresDictionary->ContainsKey(Feature))
		{
			return m_WantFeautresDictionary[Feature];
		}
		else
		{
			return false;
		}
	}

private:

	// Global exception handler. Is called when there is an unhandled exception in the plugin
	void CurrentDomain_UnhandledException(Object^ sender, UnhandledExceptionEventArgs^ e)
	{
		System::Windows::Forms::MessageBox::Show("Critical Error in plugin "+this->GetType()->ToString()+"! FARR will terminate: " + e->ExceptionObject->ToString());
	}

	// The DCUpdate file name is used to get variables from there
	String^ m_DCUpdateFile;

	// The features the user can request for his plugin
	Dictionary<EWantFeatures, bool>^ m_WantFeautresDictionary;

	// The parameters that cause the plugin to be invoked
	String^ m_DefaultRegex;
	String^ m_DefaultRegexFilter;
	String^ m_DefaultKeyword;
	String^ m_DefaultScore;
	String^ m_DefaultAlias;

	// The general info parameters
	String^ m_DisplayName;
	String^ m_VersionString;
	String^ m_ReleaseDateString;
	String^ m_Author;
	String^ m_HomepageURL;
	String^ m_UpdateURL;
	String^ m_ShortDescription;
	String^ m_LongDescription;
	String^ m_AdvConfigString;
	String^ m_ReadMeString;
	String^ m_IconFilename;


};



}

using namespace FARRPlugin;

// The RCW that forwards the DLL calls from the raw dll to the C# version
// It is also responsible on using reflection to load the object.
private ref class FARRPluginRCW
{
public:

	static FARRPluginBase^ PluginRCW = nullptr;

	// Used to dynamically load the assemblies
	static String^ AssemblyFolder = nullptr;

	static BOOL MyPlugin_DoInitRCW()
	{
		LoadPlugin();
		return PluginRCW->Init();
	}

	static Assembly^ OnAssemblyResolve(Object^ sender, ResolveEventArgs^ args)
	{
		String^ AsmName = gcnew String(AssemblyFolder+args->Name->Split(',')[0]);
		AsmName += ".dll";
		if(System::IO::File::Exists(AsmName))
		{
			Assembly^ ass = Assembly::LoadFile(AsmName);
			return ass;
		}
		else
		{
			return nullptr;
		}
	}

	 
	static BOOL MyPlugin_DoShutdown()
	{
		return PluginRCW->Shutdown();
	}

	static BOOL MyPlugin_DoAdvConfig()
	{
		return PluginRCW->AdvConfig();
	}

	static BOOL MyPlugin_DoShowReadMe()
	{
		return PluginRCW->ShowReadMe();
	}

	static void EFuncName_Inform_WindowIsHiding(HWND hwndp)
	{
		return PluginRCW->WindowIsHiding((IntPtr)hwndp);
	}

	static void EFuncName_Inform_WindowIsUnHiding(HWND hwndp)
	{
		return PluginRCW->WindowIsHiding((IntPtr)hwndp);
	}

	static BOOL EFuncName_Do_AdjustResultScore(const char* itempath, int *scorep)
	{
		int Score = *scorep;

		return PluginRCW->AdjustResultScore(gcnew String(itempath), Score);

		*scorep = Score;
	}

	static BOOL EFuncName_Ask_WantFeature(E_WantFeaturesT featureid)
	{
		return PluginRCW->WantFeature((FARRPluginBase::EWantFeatures)(Int32)featureid);
	}


	// Config
	static BOOL MyPlugin_GetStrVal(const char* varname,char *destbuf, int maxlen)
	{
		LoadPlugin();

		if (strcmp(varname,DEF_FieldName_RegexStr)==0)
		{
			ToCharPointer(PluginRCW->DefaultRegex, destbuf, maxlen);
			return TRUE;
		}
		if (strcmp(varname,DEF_FieldName_RegexFilterStr)==0)
		{
			ToCharPointer(PluginRCW->DefaultRegexFilter, destbuf, maxlen);
			return TRUE;
		}
		if (strcmp(varname,DEF_FieldName_KeywordStr)==0)
		{
			ToCharPointer(PluginRCW->DefaultKeyword, destbuf, maxlen);
			return TRUE;
		}
		if (strcmp(varname,DEF_FieldName_ScoreStr)==0)
		{
			ToCharPointer(PluginRCW->DefaultScore, destbuf, maxlen);
			return TRUE;
		}
		if (strcmp(varname,DEF_FieldName_AliasStr)==0)
		{
			ToCharPointer(PluginRCW->DefaultAlias, destbuf, maxlen);
			return TRUE;
		}

		// Regarding general info:
		if (strcmp(varname,DEF_FieldName_DisplayName)==0)
		{
			ToCharPointer(PluginRCW->DisplayName, destbuf, maxlen);
			return TRUE;
		}
		if (strcmp(varname,DEF_FieldName_VersionString)==0)
		{
			ToCharPointer(PluginRCW->VersionString, destbuf, maxlen);
			return TRUE;
		}
		if (strcmp(varname,DEF_FieldName_ReleaseDateString)==0)
		{
			ToCharPointer(PluginRCW->ReleaseDateString, destbuf, maxlen);
			return TRUE;
		}
		if (strcmp(varname,DEF_FieldName_Author)==0)
		{
			ToCharPointer(PluginRCW->Author, destbuf, maxlen);
			return TRUE;
		}
		if (strcmp(varname,DEF_FieldName_UpdateURL)==0)
		{
			ToCharPointer(PluginRCW->UpdateURL, destbuf, maxlen);
			return TRUE;
		}
		if (strcmp(varname,DEF_FieldName_HomepageURL)==0)
		{
			ToCharPointer(PluginRCW->HomepageURL, destbuf, maxlen);
			return TRUE;
		}
		if (strcmp(varname,DEF_FieldName_ShortDescription)==0)
		{
			ToCharPointer(PluginRCW->ShortDescription, destbuf, maxlen);
			return TRUE;
		}
		if (strcmp(varname,DEF_FieldName_LongDescription)==0)
		{
			ToCharPointer(PluginRCW->LongDescription, destbuf, maxlen);
			return TRUE;
		}

		// Do we support advanced options? Return "" if not
		if (strcmp(varname,DEF_FieldName_AdvConfigString)==0)
		{
			ToCharPointer(PluginRCW->AdvConfigString, destbuf, maxlen);
			return TRUE;
		}
		// Do we support a readme? Return "" if not
		if (strcmp(varname,DEF_FieldName_ReadMeString)==0)
		{
			ToCharPointer(PluginRCW->ReadMeString, destbuf, maxlen);
			return TRUE;
		}

		// Icon filename for plugin?  We pass icons by icon filename
		if (strcmp(varname,DEF_FieldName_IconFilename)==0)
		{
			ToCharPointer(PluginRCW->IconFilename, destbuf, maxlen);
			return TRUE;
		}

		// not found
		return FALSE;
	}



	static void MyPlugin_SetStrVal(const char* varname, void *val)
	{
		// farr host will pass us function pointer we will call
		if (strcmp(varname,DEF_FieldName_NotifySearchCallbackFp)==0)
		{
			callbackfp_notifysearchstatechanged = (Fp_GlobalPluginCallback_NotifySearchStateChanged)val;
		}
	}
	// Searching
	static BOOL EFuncName_Inform_SearchBegins(const char* searchstring_raw, const char *searchstring_lc_nokeywords, BOOL explicitinvocation)
	{
		return PluginRCW->SearchBegins(gcnew String(searchstring_raw), gcnew String(searchstring_lc_nokeywords), explicitinvocation);
	}

	static BOOL EFuncName_Inform_RegexSearchMatch(const char* searchstring_raw, const char *searchstring_lc_nokeywords, int regexgroups, char** regexcharps)
	{
		regexgroups++;

		array<String^>^ RegexGroups = gcnew array<String^>(regexgroups);
		for(int i = 0; i < regexgroups; i++)
		{
			RegexGroups[i] = gcnew String(regexcharps[i]);
		}

		return PluginRCW->RegexSearchMatch(gcnew String(searchstring_raw), gcnew String(searchstring_lc_nokeywords), regexgroups, RegexGroups);
	}
	static void EFuncName_Inform_SearchEnds()
	{
		PluginRCW->SearchEnds();
	}


	// Results status
	static E_SearchStateT EFuncName_Ask_SearchState()
	{
		return (E_SearchStateT)PluginRCW->SearchState;
	}

	static E_ResultAvailableStateT EFuncName_Ask_IsAnyResultAvailable()
	{
		return (E_ResultAvailableStateT)PluginRCW->ResultsAvailableState;
	}
	static int EFuncName_Ask_HowManyItemResultsAreReady()
	{
		return PluginRCW->ResultsAmount;
	}

	static BOOL EFuncName_Request_LockResults(BOOL dolock)
	{
		return PluginRCW->Request_LockResults((bool)dolock);
	}
	// Found items
	static BOOL EFuncName_Request_ItemResultByIndex(int resultindex, char *destbuf_path, char *destbuf_caption, char *destbuf_groupname, char *destbuf_iconfilename, void** tagvoidpp, int maxlen, E_ResultPostProcessingT *resultpostprocmodep, int *scorep, E_EntryTypeT *entrytypep)
	{
		// Anyone knows what this does??
		*tagvoidpp=NULL;

		String^ Path = gcnew String("");
		String^ Caption = gcnew String("");
		String^ Group = gcnew String("");
		String^ IconFile = gcnew String("");
		FARRPluginBase::EEntryType EntryType;
		FARRPluginBase::EResultPostProcessing ResultPostProcessing;
		Int32 ScoreChange = 0;

 		PluginRCW->GetItemResultByIndex(resultindex, Path, Caption, Group, IconFile, ResultPostProcessing, ScoreChange, EntryType);

		ToCharPointer(Path, destbuf_path, maxlen);
		ToCharPointer(Caption, destbuf_caption, maxlen);
		ToCharPointer(Group, destbuf_groupname, maxlen);
		ToCharPointer(IconFile, destbuf_iconfilename, maxlen);
		*resultpostprocmodep = (E_ResultPostProcessingT)ResultPostProcessing;
		*entrytypep = (E_EntryTypeT)EntryType;

		return TRUE;
	}

	static BOOL EFuncName_Allow_ProcessTrigger(const char* destbuf_path, const char* destbuf_caption, const char* destbuf_groupname, 
										int pluginid,int thispluginid, int score, E_EntryTypeT entrytype, void* tagvoidp, 
										BOOL *closeafterp)
	{
		System::Boolean CloseAfterOperation;

		BOOL RetVal = PluginRCW->ProcessTrigger(gcnew String(destbuf_path), gcnew String(destbuf_caption), gcnew String(destbuf_groupname), 
			pluginid, thispluginid, score, (FARRPluginBase::EEntryType)entrytype, CloseAfterOperation);

		*closeafterp = (BOOL)(Int32)CloseAfterOperation;

		return RetVal;					
	}



private:

	static void MsgBox(String^ Message)
	{
		System::Windows::Forms::MessageBox::Show(Message);
	}

	static void ToCharPointer(String^ Source, char* Destination, int MaxLen)
	{
		strncpy(Destination,(char*)Marshal::StringToHGlobalAnsi(Source).ToPointer(),MaxLen);
	}

	// Loads the plugin assembly if not loaded already
	static bool m_LoadedPlugin = false;
	static void LoadPlugin()
	{
		if(m_LoadedPlugin == true)
			return;

		try
		{
			//MsgBox("Attach");
			AssemblyFolder = System::IO::Path::GetDirectoryName(Assembly::GetExecutingAssembly()->Location) + "\\";
			AppDomain^ app = AppDomain::CurrentDomain;
			app->AssemblyResolve += gcnew ResolveEventHandler(OnAssemblyResolve);

			String^ PluginTypeName = "";
			Assembly^ PluginAsm = GetPluginAssembly(PluginTypeName);
			if(PluginAsm == nullptr)
			{
				throw gcnew Exception("Unable to load the plugin C# assembly");
			}

			PluginRCW = (FARRPluginBase^)PluginAsm->CreateInstance(PluginTypeName);

			m_LoadedPlugin = true; 
		}
		catch(Exception^ e) 
		{
			MsgBox(e->ToString());
		}
	}

	static Assembly^ GetPluginAssembly(String^% PluginTypeName)
	{
		Assembly^ PluginAssembly = nullptr;

		array<String^>^ Files = Directory::GetFiles(AssemblyFolder, "*.dll");
        for(int i = 0; i < Files->Length; i++)
        {
			Assembly^ CurrentAssembly = Assembly::LoadFile(Files[i]);
			array<Type^>^ Types = CurrentAssembly->GetTypes();
			for(int j = 0; j < Types->Length; j++)
			{
				Type^ CurType = Types[j];
				if(CurType->BaseType != nullptr && CurType->BaseType == FARRPluginBase::typeid)
				{
					if(PluginAssembly == nullptr)
					{
						PluginAssembly = CurrentAssembly;
						PluginTypeName = CurType->FullName;
					}
					else
					{
						throw gcnew Exception("More than one file with C# FARR plugin were found! Directory: " + AssemblyFolder);
					}
				}
			}
        }

		return PluginAssembly;
	}


};


#endif