﻿using System;
using System.Collections.Generic;
using System.Text;
using FARRPlugin;
using iTunesLib;
using System.Drawing;
using System.Reflection;
using System.Drawing.Imaging;
using System.IO;

namespace FARRTunes.Actions
{
    abstract class ActionItem
    {
        public ActionItem()
        {
        }

        public ActionItem(string Icon, string Shortcut)
        {
            if (Shortcut != "")
            {
                m_Shortcut = " (" + Shortcut + ")";
            }

            this.Icon = Icon;
        }

        public void GetData(ref string Caption, ref string Group, ref string PathItem, ref string IconFile)
        {
            Caption = this.Caption;

            GetDataRaw(ref Group, ref PathItem);

            IconFile = GetFilePluginPath(m_Icon);
        }
        protected abstract void GetDataRaw(ref string Group, ref string PathItem);

        public abstract void Execute(ref bool HideOnExecute);

        string m_Icon = "";
        protected string Icon
        {
            set
            {
                m_Icon = value;
            }
        }

        // Returns the given file with the full plugin path (e.g 1.txt turns to C:\Program Files\FARR\Plugins\FARRTunes\1.bat)
        protected string GetFilePluginPath(string File)
        {
            return Path.Combine(System.IO.Path.GetDirectoryName(Assembly.GetCallingAssembly().Location), File);
        }

        string m_Shortcut = "";
        // Used by the list to find the selected item
        public string Caption
        {
            get
            {
                return CaptionRaw + m_Shortcut;
            }
        }
        public abstract string CaptionRaw { get; }
    }

    class SearchChanger : ActionItem
    {
        string m_SearchValue;
        public string SearchValue 
        {
            get { return m_SearchValue; } 
            private set { m_SearchValue = value; } 
        }

        string m_Caption;
        public SearchChanger(string Caption, string SearchValue, string Icon, string Shortcut)
            : base(Icon, Shortcut)
        {
            m_Caption = Caption;
            this.SearchValue = SearchValue;
        }

        protected override void GetDataRaw(ref string Group, ref string PathItem)
        {
        }

        public override void Execute(ref bool HideOnExecute)
        {
            HideOnExecute = false;
            FARRTunes.Instance.SetSearch(this.SearchValue + " ");
        }

        public override string CaptionRaw
        {
            get { return m_Caption; }
        }
    }

    abstract class TunesItem : ActionItem
    {
        protected TunesItem()
            : base()
        {

        }

        protected TunesItem(string Icon, string Shortcut)
            : base(Icon, Shortcut)
        {

        }

        protected override void GetDataRaw(ref string Group, ref string PathItem)
        {
        }

        public override void Execute(ref bool HideOnExecute)
        {
            throw new NotImplementedException();
        }
    }

    class TunesPlayPause : TunesItem
    {
        public TunesPlayPause()
            : base("", "p")
        {
        }

        protected override void GetDataRaw(ref string Group, ref string PathItem)
        {
            iTunesAppClass TunesCom = TunesHelper.GetTunesCom();

            if (TunesCom.PlayerState == ITPlayerState.ITPlayerStatePlaying)
            {
                this.Icon = GetFilePluginPath("pause.ico");
            }
            else if (TunesCom.PlayerState == ITPlayerState.ITPlayerStateStopped)
            {
                this.Icon = GetFilePluginPath("play.ico");
            }
        }

        public override void Execute(ref bool HideOnExecute)
        {
            iTunesAppClass TunesCom = TunesHelper.GetTunesCom();

            if (TunesCom.PlayerState == ITPlayerState.ITPlayerStatePlaying)
            {
                TunesCom.Pause();
            }
            else if (TunesCom.PlayerState == ITPlayerState.ITPlayerStateStopped)
            {
                TunesCom.Play();
            }
        }

        public override string CaptionRaw
        {
            get
            {
                iTunesAppClass TunesCom = TunesHelper.GetTunesCom();

                if (TunesCom.PlayerState == ITPlayerState.ITPlayerStatePlaying)
                {
                    return "Pause";
                }
                else if (TunesCom.PlayerState == ITPlayerState.ITPlayerStateStopped)
                {
                    return "Play";
                }
                else
                {
                    throw new InvalidDataException("Expected status is 'Playing' or 'Stopped'. Received: " +
                                                   TunesCom.PlayerState.ToString());
                }
            }
        }
    }

    class TunesNext : TunesItem
    {
        public TunesNext()
            : base("next.ico", "nt")
        {
        }

        protected override void GetDataRaw(ref string Group, ref string PathItem)
        {
        }

        public override void Execute(ref bool HideOnExecute)
        {
            iTunesAppClass TunesCom = TunesHelper.GetTunesCom();

            TunesCom.NextTrack();
        }

        public override string CaptionRaw
        {
            get { return "Next track"; }
        }
    }

    class TunesPrevious : TunesItem
    {
        public TunesPrevious()
            : base("previous.ico", "pt")
        {
        }

        protected override void GetDataRaw(ref string Group, ref string PathItem)
        {
        }

        public override void Execute(ref bool HideOnExecute)
        {
            iTunesAppClass TunesCom = TunesHelper.GetTunesCom();

            TunesCom.PreviousTrack();
        }

        public override string CaptionRaw
        {
            get
            {
                return "Previous track";
            }
        }
    }

    class TunesSong : TunesItem
    {
        static readonly string m_TempPath = null;

        // Static initializer
        static TunesSong()
        {
            m_TempPath = Path.Combine(System.IO.Path.GetDirectoryName(Assembly.GetCallingAssembly().Location), "Temp");
            Directory.CreateDirectory(m_TempPath);
            string[] Files = Directory.GetFiles(m_TempPath);

            foreach (string FileName in Files)
            {
                File.Delete(FileName);
            }
        }

        IITTrack m_Track = null;
        const string m_Shortcut = "cur";
        bool m_IsCurrentSongMode = false;

        public TunesSong()
            : base("tune.ico", m_Shortcut)
        {
            m_IsCurrentSongMode = true;
        }

        public TunesSong(IITTrack Track)
            : base("tune.ico", m_Shortcut)
        {
            m_Track = Track;
            m_IsCurrentSongMode = false;
        }

        protected override void GetDataRaw(ref string Group, ref string PathItem)
        {
            if (m_IsDisplayingSong == true)
            {
                Group = string.Format("Rating: {0}. Play count: {1}. Last played: {2}.",
                                       m_Track.Rating / 20, m_Track.PlayedCount, m_Track.PlayedDate);

                if (m_IsCurrentSongMode == true && m_Track.Artwork.Count > 0)
                {
                    this.Icon = GetTuneIcon(m_Track);
                }
            }
        }

        private string GetTuneIcon(IITTrack Track)
        {
            string Extension = "";
            switch (Track.Artwork[1].Format)
            {
                case ITArtworkFormat.ITArtworkFormatBMP:
                    Extension = "bmp";
                    break;
                case ITArtworkFormat.ITArtworkFormatJPEG:
                    Extension = "jpg";
                    break;
                case ITArtworkFormat.ITArtworkFormatPNG:
                    Extension = "png";
                    break;
                default:
                    break;
            }

            string FileName = Path.Combine(m_TempPath, Path.GetRandomFileName() + Extension);
            string IconFile = FileName + ".ico";

            DeleteFile(FileName);
            DeleteFile(IconFile);

            Track.Artwork[1].SaveArtworkToFile(FileName);

            Bitmap BP = new Bitmap(FileName);
            ResizeBitmap(BP, 32, 32).Save(IconFile, ImageFormat.Bmp);
            return IconFile;
        }

        public override void Execute(ref bool HideOnExecute)
        {
            HideOnExecute = false;

            if (m_IsDisplayingSong == true)
            {
                if (m_IsCurrentSongMode == true)
                {
                    FARRTunes.Instance.SetSearch(Constants.CurrentTune + " ");
                }
                else
                {
                    HideOnExecute = true;
                    m_Track.Play();
                }
            }
        }

        private static void DeleteFile(string FileName)
        {
            if (File.Exists(FileName))
            {
                File.Delete(FileName);
            }
        }

        private Bitmap ResizeBitmap(Bitmap b, int nWidth, int nHeight)
        {
            Bitmap result = new Bitmap(nWidth, nHeight);
            using (Graphics g = Graphics.FromImage((Image)result))
                g.DrawImage(b, 0, 0, nWidth, nHeight);
            return result;
        }

        bool m_IsDisplayingSong = false;
        public override string CaptionRaw
        {
            get
            {
                if (m_IsCurrentSongMode == true)
                {
                    iTunesAppClass TunesCom = TunesHelper.GetTunesCom();

                    m_Track = TunesCom.CurrentTrack;
                }

                if (m_Track != null)
                {
                    m_IsDisplayingSong = true;
                    return m_Track.Artist + " - " + m_Track.Name;
                }
                else
                {
                    m_IsDisplayingSong = false;
                    return "iTunes stopped - Play a song or a playlist";
                }
            }
        }
    }

    class TunesPlaylist : TunesItem
    {
        IITPlaylist m_Playlist = null;

        public TunesPlaylist(IITPlaylist Playlist)
            : base("playlist.ico", "")
        {
            m_Playlist = Playlist;
        }

        protected override void GetDataRaw(ref string Group, ref string PathItem)
        {
            Group = string.Format("Contains {0} songs", m_Playlist.Tracks.Count);
        }

        public override void Execute(ref bool HideOnExecute)
        {
            m_Playlist.PlayFirstTrack();
        }

        public override string CaptionRaw
        {
            get { return m_Playlist.Name; }
        }
    }

    class LyricsItem : TunesItem
    {
        protected override void GetDataRaw(ref string Group, ref string PathItem)
        {
            IITFileOrCDTrack Track = TunesHelper.GetTunesCom().CurrentTrack as IITFileOrCDTrack;

            FARRTunes.Instance.ExecuteFARRCommand("window.richeditmode", Track.Lyrics);
            FARRTunes.Instance.ExecuteFARRCommand("statusbar", "Lyrics");
        }

        public override string CaptionRaw
        {
            get { return "Lyrics"; }
        }
    }

    class RateSongItem : TunesItem
    {
        int m_Rating = 0;
        public RateSongItem(int Rating)
            : base("rt" + Rating.ToString() + ".ico", "rt" + Rating.ToString())
        {
            m_Rating = Rating;
        }

        public override void Execute(ref bool HideOnExecute)
        {
            TunesHelper.GetTunesCom().CurrentTrack.Rating = m_Rating * 20;
        }

        public override string CaptionRaw
        {
            get
            {
                switch (m_Rating)
                {
                    case 0:
                        return "No rating";
                    default:
                        return "Rate " + m_Rating.ToString() + " stars";
                }
            }
        }
    }
}
