using System.Collections.Generic;
using FARRPlugin;
using iTunesLib;
using System.IO;
using System.Drawing;
using System.Drawing.Imaging;
using System.Reflection;
using System.Text.RegularExpressions;
using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace FARRTunes
{
    public class FARRTunes : FARRPluginBase
    {
        #region Metadata

        protected override void SetGeneralInfoParameters(ref string DisplayName, ref string VersionString, ref string ReleaseDateString,
                                                         ref string Author, ref string HomepageURL, ref string UpdateURL,
                                                         ref string ShortDescription, ref string LongDescription, ref string AdvConfigString,
                                                         ref string ReadMeString, ref string IconFilename)
        {
            DisplayName = "FARR iTunes";
            VersionString = GetValueFromDCUpdate(EDCUpdateValues.Version);
            ReleaseDateString = "Dec 25rd, 2007";
            Author = "Vitaly Belman (Icons by: FastIcon.com)";
            HomepageURL = GetValueFromDCUpdate(EDCUpdateValues.WebPage);
            UpdateURL = GetValueFromDCUpdate(EDCUpdateValues.UpdateFile);
            ShortDescription = "Control iTunes from FARR";
            LongDescription = "Control iTunes from FARR";
            IconFilename = "farrtunes.ico";
        }

        protected override void SetSearchParameters(ref string DefaultAlias, ref string DefaultRegex, ref string DefaultRegexFilter, ref string DefaultKeyword, ref string DefaultScore)
        {
            DefaultAlias = Constants.Tunes;
            DefaultScore = "300";
        }
        protected override void SetFeatureParamaters(Dictionary<EWantFeatures, bool> FeatureDictionary)
        {
            FeatureDictionary[EWantFeatures.SearchInputExplicit] = true;
            FeatureDictionary[EWantFeatures.TryHandleTrigger] = true;
        }

        #endregion
        #region Internal mechanics

        public override void SearchEnds()
        {
            ClearResults();
        }
        public override bool LockResults()
        {

            return true;
        }
        public override bool UnlockResults()
        {
            ClearResults();
            SearchStateChanged();

            return true;
        }

        #endregion

        static FARRTunes m_Instance = null;
        static public FARRTunes Instance { get { return m_Instance; } }

        public FARRTunes()
        {
            m_Instance = this;
            ReloadActionLists();
        }

        private void ReloadActionLists()
        {
            m_ActionLists.Clear();
            // Loads all the action lists to a list
            Type[] Types = this.GetType().Assembly.GetTypes();
            foreach (Type CurType in Types)
            {
                if (CurType.IsSubclassOf(typeof(Lists.ActionsList)) && CurType.IsAbstract == false)
                {
                    m_ActionLists.Add((Lists.ActionsList)Activator.CreateInstance(CurType));
                }
            }
        }

        public override bool SearchBegins(string SearchStringRaw, string SearchStringKeyKeywords, bool ExplicitInvocation)
        {
            try
            {
                ClearTemporaryItems();
                m_CurrentList = null;

                m_ResultsAvailableState = EResultsAvailableState.ItemResults;
                foreach (Lists.ActionsList List in m_ActionLists)
                {
                    Regex ActivatorRegex = new Regex(List.ActivatorRegex);

                    if (ActivatorRegex.IsMatch(SearchStringRaw))
                    {
                        if (m_CurrentList == null || m_CurrentList.ActivatorRegex.Length < List.ActivatorRegex.Length)
                        {
                            m_CurrentList = List;
                        }
                    }
                }

                if (m_CurrentList != null)
                {
                    m_ResultsAmount = m_CurrentList.GetActionsCount(SearchStringRaw);

                    if (m_ResultsAmount > 0)
                    {
                        m_ResultsAvailableState = m_CurrentList.ResultsType;
                    }
                    else
                    {
                        m_ResultsAvailableState = EResultsAvailableState.None;
                    }

                    SearchStateChanged();
                }

                return true;
            }
            catch (COMException e)
            {
                OnTunesConnectionLost();

                return false;
            }
            catch (Exception e)
            {
                OnUnhandledException(e);

                return false;
            }
        }

        private void OnTunesConnectionLost()
        {
            MessageBox.Show("Lost connection to iTunes application. Reconnecting...", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            ReloadActionLists();
            SetSearch(Constants.Tunes);
        }

        public override void GetItemResultByIndex(int ResultIndex, ref string PathItem, ref string Caption, ref string Group, ref string IconFile, ref FARRPluginBase.EResultPostProcessing ResultPostProcessing, ref int ScoreChange, ref FARRPluginBase.EEntryType EntryType)
        {
            try
            {
                m_CurrentList[ResultIndex].GetData(ref Caption, ref Group, ref PathItem, ref IconFile);

                ResultPostProcessing = EResultPostProcessing.ImmediateDisplay;
                EntryType = EEntryType.File;

            }
            catch (COMException e)
            {
                OnTunesConnectionLost();
            }
            catch (Exception e)
            {
                OnUnhandledException(e);
            }
        }

        private static void OnUnhandledException(Exception e)
        {
            MessageBox.Show("Unhandled exception, please alert the author of the following information \r\n\r\n" + e.ToString(),
                "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        public override bool ProcessTrigger(string Path, string Caption, string Group, int PluginId, int ThisPlugId, int Score, EEntryType EntryType, out bool CloseAfterOperation)
        {
            if (PluginId == ThisPlugId)
            {
                CloseAfterOperation = true;
                m_CurrentList.FindByCaption(Caption).Execute(ref CloseAfterOperation);

                return true;
            }
            else
            {
                CloseAfterOperation = false;
                return false;
            }
        }


        private void ClearTemporaryItems()
        {
            foreach (Lists.ActionsList List in m_ActionLists)
            {
                List.ClearTemporaryItems();
            }
        }

        private List<Lists.ActionsList> m_ActionLists = new List<Lists.ActionsList>();
        private Lists.ActionsList m_CurrentList = null;
    }
}
