#include "StdAfx.h"
#include "ServiceManager.h"
#include "JrPlugin_GenericShell.h"

void ServiceManager::SleepWaitHint(DWORD dwWaitHint)
{
  // Do not wait longer than the wait hint. A good interval is 
  // one tenth the wait hint, but no less than 1 second and no 
  // more than 3 seconds. 
  DWORD dwWaitTime = dwWaitHint / 10;

  if( dwWaitTime < 1000 )
    dwWaitTime = 1000;
  else if ( dwWaitTime > 3000 )
    dwWaitTime = 3000;

  Sleep( dwWaitTime );
}

void ServiceManager::LogError(const TCHAR* funcName, DWORD dwErrorCode)
{
#ifndef SERVICEHELPER  
  LPVOID lpvMsgBuf;
  TCHAR szErrorDesc[1024] = {0};

  DWORD dwResult = FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_ALLOCATE_BUFFER,
    0, 
    dwErrorCode, 
    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
    (LPTSTR)&lpvMsgBuf, 
    0, 
    0);

  if (!dwResult)
  {
    _stprintf(szErrorDesc, _T("Function: %s, Win32 Error: %d, Unknown Error"), funcName, dwErrorCode);
  }
  else
  {
    _stprintf(szErrorDesc, _T("Function: %s, Win32 Error: %d Description: %s"), funcName, dwErrorCode, lpvMsgBuf);
    LocalFree(lpvMsgBuf);
  }

  ReportErrorToHost(szErrorDesc);
#endif  
}

bool ServiceManager::GetServices(CServiceList& services, int status_mask)
{
  services.clear();

  //form the mask
  int mask = 0;
  if ((status_mask & SVC_STATUS_ACTIVE) && (status_mask & SVC_STATUS_INACTIVE))
  {
    mask = SERVICE_STATE_ALL;
  }
  else if (status_mask & SVC_STATUS_ACTIVE)
  {
    mask = SERVICE_ACTIVE;
  }
  else if (status_mask & SVC_STATUS_INACTIVE)
  {
    mask = SERVICE_INACTIVE;
  }
  else
  {
    return false;
  }

  SC_HANDLE schSCManager = ::OpenSCManager(NULL, NULL, SC_MANAGER_ENUMERATE_SERVICE);  
  if (schSCManager == NULL) 
  { 
    LogError(_T("OpenSCManager"), GetLastError());
    return false;
  }

  ENUM_SERVICE_STATUS service;    
  DWORD dwBytesNeeded = 0;
  DWORD dwServicesReturned = 0;
  DWORD dwResumedHandle = 0;
  DWORD dwServiceType = SERVICE_WIN32;

  // Query services
  BOOL retVal = EnumServicesStatus(schSCManager, dwServiceType, mask, 
    &service, sizeof(ENUM_SERVICE_STATUS), &dwBytesNeeded, &dwServicesReturned,
    &dwResumedHandle);

  if (!retVal) 
  {
    // Need big buffer
    if (ERROR_MORE_DATA == ::GetLastError()) 
    {
      // Set the buffer
      DWORD dwBytes = sizeof(ENUM_SERVICE_STATUS) + dwBytesNeeded;
      ENUM_SERVICE_STATUS* pServices = NULL;
      pServices = new ENUM_SERVICE_STATUS [dwBytes];

      // Now query again for services
      EnumServicesStatus(schSCManager, dwServiceType, 
        mask, pServices, dwBytes, &dwBytesNeeded, 
        &dwServicesReturned, &dwResumedHandle);            

      // now traverse each service to get information
      for (unsigned iIndex = 0; iIndex < dwServicesReturned; iIndex++) 
      {
        //fill the array
        CService s;
        s.name = pServices[iIndex].lpServiceName;
        s.display_name = pServices[iIndex].lpDisplayName;        
        s.status = (pServices[iIndex].ServiceStatus.dwCurrentState == SERVICE_STOPPED) ? SVC_STATUS_INACTIVE : SVC_STATUS_ACTIVE;
        services.push_back(s);        
      }

      delete [] pServices;
      pServices = NULL;
    }    
    else 
    {
      LogError(_T("EnumServicesStatus"), GetLastError());
    }
  }

  CloseServiceHandle(schSCManager);

  return true;
}

SVCERR ServiceManager::GetServiceDependencies(const TCHAR* srvName, CServiceList& services)
{
  services.clear();

  SC_HANDLE schSCManager  = NULL;
  SC_HANDLE schService    = NULL;

  if (!GetServiceHandle(srvName, GENERIC_READ, SERVICE_ENUMERATE_DEPENDENTS | SERVICE_QUERY_STATUS, &schSCManager, &schService))
  {
    return SVC_FAIL;
  } 

  DWORD i, dwBytesNeeded, dwCount;
  LPENUM_SERVICE_STATUS   lpDependencies = NULL;

  // Pass a zero-length buffer to get the required buffer size
  if (EnumDependentServices(schService, SERVICE_ACTIVE, lpDependencies, 0, &dwBytesNeeded, &dwCount)) {
    // If the Enum call succeeds, then there are no dependent
    // services so do nothing    
  } 
  else 
  {
    if (GetLastError() != ERROR_MORE_DATA ) return SVC_FAIL; // Unexpected error
    // Allocate a buffer for the dependencies
    lpDependencies = (LPENUM_SERVICE_STATUS) HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, dwBytesNeeded);
    if (!lpDependencies ) return SVC_FAIL;

    // Enumerate the dependencies
    if ( !EnumDependentServices( schService, SERVICE_ACTIVE,
      lpDependencies, dwBytesNeeded, &dwBytesNeeded, &dwCount)) 
    {
      HeapFree( GetProcessHeap(), 0, lpDependencies );    
      return SVC_FAIL;
    }

    for ( i = 0; i < dwCount; i++ ) 
    {
      // Open the service
      ENUM_SERVICE_STATUS   ess = *(lpDependencies + i);

      CService svc;
      svc.name = ess.lpServiceName;        
      svc.display_name = ess.lpDisplayName;
      svc.status = SVC_STATUS_UNKNOWN;

      services.push_back(svc);
    }  
    HeapFree( GetProcessHeap(), 0, lpDependencies );
  }
  return SVC_SUCCESS;
}

bool ServiceManager::ChangeServiceStartMode(const TCHAR* srvName, DWORD newStartType)
{ 
  bool bRet = false;

  SC_HANDLE schSCManager  = NULL;
  SC_HANDLE schService    = NULL;

  if (GetServiceHandle(srvName, GENERIC_WRITE, SERVICE_CHANGE_CONFIG, &schSCManager, &schService))
  {    
    if(!ChangeServiceConfig(schService, SERVICE_NO_CHANGE, newStartType, SERVICE_NO_CHANGE, NULL, NULL, NULL, NULL, NULL, NULL, NULL))
    {   
      bRet = false;
    }
    else
    { 
      bRet = true;
    }
  }

  CloseServiceHandle(schSCManager);
  CloseServiceHandle(schService);

  return bRet;  
}

bool ServiceManager::IsServiceDisabled(const TCHAR* srvName)
{
  bool bRet = false;

  SC_HANDLE schSCManager  = NULL;
  SC_HANDLE schService    = NULL;
  DWORD dwBytesNeeded     = 0;

  if (GetServiceHandle(srvName, GENERIC_READ, SERVICE_QUERY_CONFIG, &schSCManager, &schService))
  {
    LPQUERY_SERVICE_CONFIG lpqscBuf;  
    lpqscBuf = (LPQUERY_SERVICE_CONFIG) LocalAlloc(LPTR, 4096); 
    if (lpqscBuf == NULL) return SVC_FAIL;  

    if (QueryServiceConfig(schService, lpqscBuf, 4096, &dwBytesNeeded) ) 
    {
      if (lpqscBuf->dwStartType == SERVICE_DISABLED) 
      {
        bRet = true;
      }
    }

    LocalFree(lpqscBuf);
    CloseServiceHandle(schSCManager);
    CloseServiceHandle(schService);
  }     

  return bRet;
}

SVCERR ServiceManager::StartService(const TCHAR* srvName, int timeout)
{
  SC_HANDLE schSCManager  = NULL;
  SC_HANDLE schService    = NULL;
  SERVICE_STATUS_PROCESS ssStatus;
  DWORD dwOldCheckPoint   = 0; 
  DWORD dwStartTickCount  = 0;
  DWORD dwWaitTime        = 0;
  DWORD dwStatus          = 0; 
  DWORD dwBytesNeeded     = 0;
  BOOL  bTimedOut         = FALSE;

  if (!GetServiceHandle(srvName, SC_MANAGER_ALL_ACCESS, SERVICE_START | SERVICE_QUERY_STATUS | SERVICE_QUERY_CONFIG | SERVICE_CHANGE_CONFIG, &schSCManager, &schService))
  {
    return SVC_FAIL;
  }

  LPQUERY_SERVICE_CONFIG lpqscBuf;  
  lpqscBuf = (LPQUERY_SERVICE_CONFIG) LocalAlloc(LPTR, 4096); 
  if (lpqscBuf == NULL) return SVC_FAIL;  

  if (QueryServiceConfig(schService, lpqscBuf, 4096, &dwBytesNeeded) ) 
  {
    if (lpqscBuf->dwStartType == SERVICE_DISABLED) 
    {
      int res = MessageBox(NULL, _T("Service is currently disabled. Do you want to set the start type to \"Manual\" and enable it?"), _T("Confirmation"), MB_YESNO);
      if (res == IDYES)
      {
        ChangeServiceConfig(schService,SERVICE_NO_CHANGE,SERVICE_DEMAND_START,SERVICE_NO_CHANGE,NULL,              
          NULL,NULL,NULL,NULL,NULL,NULL);
      }
      else
      {
        return SVC_SUCCESS;
      }
    }
  }
  LocalFree(lpqscBuf);

  SERVICE_STATUS ss = {0};

  // Make sure the service is not already stopped
  if (!QueryServiceStatus(schService, &ss)) 
  {
    LogError(_T("QueryServiceStatus"), GetLastError());
    return SVC_FAIL;
  }
  if (ss.dwCurrentState == SERVICE_RUNNING) return SVC_SUCCESS;

  int startTime = GetTickCount();

  // If a start is pending, just wait for it
  while ( ss.dwCurrentState == SERVICE_START_PENDING) 
  {
    SleepWaitHint( ss.dwWaitHint );

    if (!QueryServiceStatus( schService, &ss))  return SVC_FAIL;
    if (ss.dwCurrentState == SERVICE_RUNNING) return SVC_SUCCESS;
    if (GetTickCount() - startTime > timeout ) return SVC_TIMEDOUT;
  }

  if (!::StartService(
    schService,  // handle to service 
    0,           // number of arguments 
    NULL) )      // no arguments 
  {
    LogError(_T("StartService"), GetLastError());
    return SVC_FAIL;
  }

  // Check the status until the service is no longer start pending. 

  if (!QueryServiceStatusEx( 
    schService,             // handle to service 
    SC_STATUS_PROCESS_INFO, // info level
    (LPBYTE)&ssStatus,              // address of structure
    sizeof(SERVICE_STATUS_PROCESS), // size of structure
    &dwBytesNeeded ) )              // if buffer too small
  {
    LogError(_T("QueryServiceStatusEx"), GetLastError());
    return SVC_FAIL;
  }

  // Save the tick count and initial checkpoint.

  dwStartTickCount = GetTickCount();
  dwOldCheckPoint = ssStatus.dwCheckPoint;

  while (ssStatus.dwCurrentState == SERVICE_START_PENDING) 
  { 
    SleepWaitHint( ssStatus.dwWaitHint );

    // Check the status again. 

    if (!QueryServiceStatusEx( 
      schService,             // handle to service 
      SC_STATUS_PROCESS_INFO, // info level
      (LPBYTE)&ssStatus,              // address of structure
      sizeof(SERVICE_STATUS_PROCESS), // size of structure
      &dwBytesNeeded ) )              // if buffer too small
      break; 

    if ( ssStatus.dwCheckPoint > dwOldCheckPoint )
    {
      // The service is making progress.

      dwStartTickCount = GetTickCount();
      dwOldCheckPoint = ssStatus.dwCheckPoint;
    }
    else
    {
      if(GetTickCount()-dwStartTickCount > ssStatus.dwWaitHint)
      {
        // No progress made within the wait hint
        bTimedOut = TRUE;
        break;
      }
    }
  } 

  CloseServiceHandle(schService); 
  CloseServiceHandle(schSCManager);

  if (ssStatus.dwCurrentState == SERVICE_RUNNING) 
  {    
    return SVC_SUCCESS;
  }
  else 
  {
    return bTimedOut ? SVC_TIMEDOUT : SVC_FAIL;
  }
}

SVCERR ServiceManager::StopService(const TCHAR* srvName, bool bStopDependencies, int timeout) 
{
  SC_HANDLE schSCManager  = NULL;
  SC_HANDLE schService    = NULL;

  if (!GetServiceHandle(srvName, SC_MANAGER_ALL_ACCESS, SERVICE_STOP | SERVICE_ENUMERATE_DEPENDENTS | SERVICE_QUERY_STATUS | SERVICE_CHANGE_CONFIG, &schSCManager, &schService))
  {
    return SVC_FAIL;
  }

  SERVICE_STATUS ss = {0};

  // Make sure the service is not already stopped
  if (!QueryServiceStatus(schService, &ss)) return SVC_FAIL;
  if (ss.dwCurrentState == SERVICE_STOPPED) return SVC_SUCCESS;

  int startTime = GetTickCount();

  // If a stop is pending, just wait for it
  while ( ss.dwCurrentState == SERVICE_STOP_PENDING) 
  {
    SleepWaitHint( ss.dwWaitHint );

    if (!QueryServiceStatus( schService, &ss))  return SVC_FAIL;
    if (ss.dwCurrentState == SERVICE_STOPPED) return SVC_SUCCESS;
    if (GetTickCount() - startTime > timeout ) return SVC_TIMEDOUT;
  }

  // If the service is running, dependencies must be stopped first
  if (bStopDependencies) 
  {
    DWORD i, dwBytesNeeded, dwCount;
    LPENUM_SERVICE_STATUS   lpDependencies = NULL;

    // Pass a zero-length buffer to get the required buffer size
    if (EnumDependentServices(schService, SERVICE_ACTIVE, lpDependencies, 0, &dwBytesNeeded, &dwCount)) {
      // If the Enum call succeeds, then there are no dependent
      // services so do nothing
    } else {
      if (GetLastError() != ERROR_MORE_DATA ) return SVC_FAIL; // Unexpected error
      // Allocate a buffer for the dependencies
      lpDependencies = (LPENUM_SERVICE_STATUS) HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, dwBytesNeeded);
      if (!lpDependencies ) return SVC_FAIL;
      __try 
      {
        // Enumerate the dependencies
        if ( !EnumDependentServices( schService, SERVICE_ACTIVE,
          lpDependencies, dwBytesNeeded, &dwBytesNeeded, &dwCount)) {
            return SVC_FAIL;
        }
        for ( i = 0; i < dwCount; i++ ) 
        {
          // Open the service
          ENUM_SERVICE_STATUS   ess = *(lpDependencies + i);
          SC_HANDLE             hDepService = OpenService(schSCManager, ess.lpServiceName,
            SERVICE_STOP | SERVICE_QUERY_STATUS );
          if ( !hDepService ) return SVC_FAIL;
          __try 
          {
            // Send a stop code
            if (!ControlService(hDepService, SERVICE_CONTROL_STOP, &ss)) return SVC_FAIL;

            // Wait for the service to stop
            while ( ss.dwCurrentState != SERVICE_STOPPED ) 
            {
              SleepWaitHint( ss.dwWaitHint );

              if ( !QueryServiceStatus( hDepService, &ss)) return SVC_FAIL;
              if (ss.dwCurrentState == SERVICE_STOPPED) break;
              if (GetTickCount() - startTime > timeout) return SVC_TIMEDOUT;
            }
          }
          __finally 
          {
            // Always release the service handle
            CloseServiceHandle( hDepService );
          }
        }
      } 
      __finally 
      {
        // Always free the enumeration buffer
        HeapFree( GetProcessHeap(), 0, lpDependencies );
      }
    } 
  }

  // Send a stop code to the main service
  if (!ControlService( schService, SERVICE_CONTROL_STOP, &ss)) return SVC_FAIL;

  // Wait for the service to stop
  while (ss.dwCurrentState != SERVICE_STOPPED) 
  {
    SleepWaitHint( ss.dwWaitHint );

    if (!QueryServiceStatus( schService, &ss)) return SVC_FAIL;
    if (ss.dwCurrentState == SERVICE_STOPPED) break;
    if (GetTickCount() - startTime > timeout) return SVC_SUCCESS;
  }

  // Return success
  return SVC_SUCCESS;
}

bool ServiceManager::GetServiceHandle(const TCHAR* svcName, DWORD scmAccess, DWORD svcAccess, SC_HANDLE* schSCManager, SC_HANDLE* schService)
{
  if (!schSCManager || !schService) return false;

  *schSCManager  = OpenSCManager(NULL, NULL, scmAccess);  
  if (!*schSCManager)
  {
    LogError(_T("GetServiceHandle"), GetLastError());
    return false;
  }

  *schService = OpenService(*schSCManager, svcName, svcAccess); 

  if (!*schService) 
  { 
    LogError(_T("OpenService"), GetLastError());
    return false;
  }
  return true;
}